from dataclasses import dataclass, asdict
import json

@dataclass
class OrderExchange:
  order_id: int
  book_id: int
  status: str = 'await_payment'

  @property
  def subject(self) -> str:
    return 'order_exchange'
  
  @property
  def message_body(self) -> str:
    return json.dumps(asdict(self))

  @classmethod
  def parse_message(cls, msg):
    data = json.loads(msg)
    return cls(**data)


from dataclasses import dataclass, asdict
import json

@dataclass
class BookSold:
  payment_id: int
  order_id: int
  buyer_id: int
  buyer_email: str
  buyer_phone: str
  sellers_id: int
  sellers_email: str
  sellers_phone: str
  book_id: int
  book_title: str
  book_author: str

  @property
  def subject(self) -> str:
    return 'book_sold'
  
  @property
  def message_body(self) -> str:
    return json.dumps(asdict(self))

  @classmethod
  def parse_message(cls, msg):
    data = json.loads(msg)
    return cls(**data)


import pika, os, time

RABBITMQ_DEFAULT_HOST = os.environ.get('RABBITMQ_DEFAULT_HOST')
RABBITMQ_DEFAULT_PORT = os.environ.get('RABBITMQ_DEFAULT_PORT')

RABBITMQ_DEFAULT_USER = os.environ.get('RABBITMQ_DEFAULT_USER')
RABBITMQ_DEFAULT_PASS = os.environ.get('RABBITMQ_DEFAULT_PASS')

def create_connection(**kwargs):
  host = kwargs['host'] if 'host' in kwargs else RABBITMQ_DEFAULT_HOST
  port = kwargs['port'] if 'port' in kwargs else RABBITMQ_DEFAULT_PORT
  user = kwargs['username'] if 'username' in kwargs else RABBITMQ_DEFAULT_USER
  passwd = kwargs['password'] if 'password' in kwargs else RABBITMQ_DEFAULT_PASS
  
  while True:
    try:
      credentials = pika.PlainCredentials(user, passwd)
      connection = pika.BlockingConnection(pika.ConnectionParameters(host, port, '/', credentials))
      print("Connected to RabbitMQ successfully.")
      return connection.channel(), connection 
    except pika.exceptions.AMQPConnectionError as e:
      print("Failed to connect to RabbitMQ. Retrying in 5 seconds...")
      print(e)
      time.sleep(5)
  



from bookshelf_common.errors.custom_error import CustomError
from bookshelf_common.errors.bad_request_error import BadRequestError
from bookshelf_common.errors.not_authorized_error import NotAuthorizedError
from bookshelf_common.errors.not_found_error import NotFoundError
from bookshelf_common.errors.request_validation_error import RequestValidationError

from bookshelf_common.events.objects.book_exchange import BookExchange
from bookshelf_common.events.objects.book_status_exchange import BookStatusExchange
from bookshelf_common.events.objects.user_exchange import UserExchange
from bookshelf_common.events.objects.order_exchange import OrderExchange
from bookshelf_common.events.objects.book_sold import BookSold
from bookshelf_common.events.objects.queue_list import QueueList

from bookshelf_common.events.base_connection import create_connection
from bookshelf_common.events.base_listener import init_listener
from bookshelf_common.events.base_listener import init_multiple_listeners
from bookshelf_common.events.base_publisher import publish_message


from bookshelf_common.errors import CustomError
class NotAuthorizedError(CustomError):
    status = 401
    message = None
    def __init__(self):
        self.message = 'Not Authorized'
        super().__init__(self.message)


from dataclasses import dataclass, asdict
import json

@dataclass
class UserExchange:
  id: int
  email: str
  first_name: str
  last_name: str
  address: str
  city: str
  phone: str
  action: str = 'create'

  @property
  def subject(self) -> str:
    return 'user_exchange'

  @property
  def message_body(self) -> str:
    return json.dumps(asdict(self))
  
  @classmethod
  def parse_message(cls, msg):
    data = json.loads(msg)
    return cls(**data)


from bookshelf_common.events import create_connection, QueueList
from typing import List
from datetime import datetime

TIME_FORMAT = "%Y-%m-%d %H:%M:%S"

def init_listener(exchange_subject, queue_subject, callback, exchange_type='fanout', channel=None, **kwargs):
  now = datetime.now()
  if channel is None:
    channel, connection = create_connection(**kwargs)
  # Create fanout exchange
  channel.exchange_declare(exchange=exchange_subject, exchange_type=exchange_type)
  
  # Create queues and bind them to the exchange
  queue = channel.queue_declare(queue=queue_subject)
  channel.queue_bind(exchange=exchange_subject, queue=queue.method.queue)

  # Set up consumers
  channel.basic_consume(queue=queue.method.queue, on_message_callback=callback)
  
  print(f'{now.strftime(TIME_FORMAT)} [*] Waiting for messages. To exit press CTRL+C')
  
  # Start consuming
  channel.start_consuming()

def init_multiple_listeners(queue_list: List[QueueList], exchange_type='fanout', channel=None, **kwargs):
  now = datetime.now()
  if channel is None:
    channel, connection = create_connection(**kwargs)

  for x in queue_list:
    # Create fanout exchange
    channel.exchange_declare(exchange=x.exchange_subject, exchange_type=exchange_type)
    
    # Create queues and bind them to the exchange
    queue = channel.queue_declare(queue=x.queue_subject)
    channel.queue_bind(exchange=x.exchange_subject, queue=queue.method.queue)

    # Set up consumers
    channel.basic_consume(queue=queue.method.queue, on_message_callback=x.callback)

  print(f'{now.strftime(TIME_FORMAT)} [*] Waiting for messages. To exit press CTRL+C')

  # Start consuming
  channel.start_consuming()



from bookshelf_common.errors import CustomError
from typing import List, Dict

class RequestValidationError(CustomError):
    status = 400
    message = None
    errors = []
    def __init__(self, errors):
        self.message = 'Invalid request parameters'
        self.errors = errors
        super().__init__(self.message)

    @property
    def serialize(self) -> List[Dict]:
        try:      
            errors = [{
                'field': x['loc'][1],
                'type': x['type'],
                'message': x['msg']
            } for x in self.errors if len(x['loc']) > 1 and x['loc'][1] is not None]
        except:
            errors = self.errors

        return errors if len(errors) > 0 else [{
            'field': 'all',
            'type': 'value_error.missing',
            'message': 'Required fields are not provided'
        }]




from setuptools import setup, find_packages

VERSION = '0.0.7.4'
DESCRIPTION = 'Share common code for bookshelf'
LONG_DESCRIPTION = 'This package will share some of the common code that we use for implementation of  bookshelf micro services.'

# Setting up
setup(
    name="bookshelf_common",
    version=VERSION,
    author="Marko Muric",
    author_email="<atis.443@gmail.com>",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    url='https://github.com/mmuric/bookshelf-common',
    packages=find_packages(),    
    project_urls = {
        "Bug Tracker": "https://github.com/mmuric/bookshelf-common/issues"
    },
    install_requires=['pika'],
    license='MIT',    
)